@@ -3,7 +3,8 @@
 #
 # ibus-anthy - The Anthy engine for IBus
 #
-# Copyright (c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -25,6 +26,10 @@ import sys
 import gobject
 import ibus
 import anthy
+from anthy import NTH_UNCONVERTED_CANDIDATE
+from anthy import NTH_KATAKANA_CANDIDATE
+from anthy import NTH_HIRAGANA_CANDIDATE
+from anthy import NTH_HALFKANA_CANDIDATE
 from tables import *
 from ibus import keysyms
 from ibus import modifier
@@ -42,6 +47,7 @@ from anthyprefs import AnthyPrefs
 from gettext import dgettext
 _  = lambda a : dgettext("ibus-anthy", a)
 N_ = lambda a : a
+UN = lambda a : unicode(a)
 
 INPUT_MODE_HIRAGANA, \
 INPUT_MODE_KATAKANA, \
@@ -66,17 +72,23 @@ CONV_MODE_PREDICTION = range(14)
 
 CLIPBOARD_RECONVERT = range(1)
 
+''' FIXME: currently configuration values are extracted by enviroment values.
+    It's better to load config.py(.in) in engine and setup instead and
+    I will move KASUMI_IMG_PATH.
+'''
+KASUMI_IMG_PATH = "/usr/share/pixmaps/kasumi.png"
+
 KP_Table = {}
-for k, v in zip(['KP_Add', 'KP_Decimal', 'KP_Divide', 'KP_Enter', 'KP_Equal',
-                 'KP_Multiply', 'KP_Separator', 'KP_Space', 'KP_Subtract'],
-                ['plus', 'period', 'slash', 'Return', 'equal',
-                 'asterisk', 'comma', 'space', 'minus']):
-    KP_Table[keysyms.__getattribute__(k)] = keysyms.__getattribute__(v)
 for s in dir(keysyms):
     if s.startswith('KP_'):
         v = keysyms.name_to_keycode(s[3:])
         if v:
             KP_Table[keysyms.name_to_keycode(s)] = v
+for k, v in zip(['KP_Add', 'KP_Decimal', 'KP_Divide', 'KP_Enter', 'KP_Equal',
+                 'KP_Multiply', 'KP_Separator', 'KP_Space', 'KP_Subtract'],
+                ['plus', 'period', 'slash', 'Return', 'equal',
+                 'asterisk', 'comma', 'space', 'minus']):
+    KP_Table[keysyms.__getattribute__(k)] = keysyms.__getattribute__(v)
 
 class Engine(ibus.EngineBase):
     __typing_mode = jastring.TYPING_MODE_ROMAJI
@@ -107,8 +119,8 @@ class Engine(ibus.EngineBase):
         self.__input_mode_activate(mode, ibus.PROP_STATE_CHECKED)
 
         mode = self.__prefs.get_value('common', 'typing_method')
-        mode = 'TypingMode.' + ['Romaji', 'Kana'][mode]
-        self.__input_mode_activate(mode, ibus.PROP_STATE_CHECKED)
+        mode = 'TypingMode.' + ['Romaji', 'Kana', 'ThumbShift'][mode]
+        self.__typing_mode_activate(mode, ibus.PROP_STATE_CHECKED)
 
         # use reset to init values
         self.__reset()
@@ -123,6 +135,9 @@ class Engine(ibus.EngineBase):
         self.__segments = list()
         self.__lookup_table.clean()
         self.__lookup_table_visible = False
+        self._MM = 0
+        self._SS = 0
+        self._H = 0
 
     def __init_props(self):
         anthy_props = ibus.PropList()
@@ -131,25 +146,25 @@ class Engine(ibus.EngineBase):
         input_mode_prop = ibus.Property(key=u"InputMode",
                                         type=ibus.PROP_TYPE_MENU,
                                         label=u"あ",
-                                        tooltip=_(u"Switch input mode"))
+                                        tooltip=UN(_("Switch input mode")))
         self.__prop_dict[u"InputMode"] = input_mode_prop
 
         props = ibus.PropList()
         props.append(ibus.Property(key=u"InputMode.Hiragana",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Hiragana")))
+                                   label=UN(_("Hiragana"))))
         props.append(ibus.Property(key=u"InputMode.Katakana",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Katakana")))
+                                   label=UN(_("Katakana"))))
         props.append(ibus.Property(key=u"InputMode.HalfWidthKatakana",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Half width katakana")))
+                                   label=UN(_("Half width katakana"))))
         props.append(ibus.Property(key=u"InputMode.Latin",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Latin")))
+                                   label=UN(_("Latin"))))
         props.append(ibus.Property(key=u"InputMode.WideLatin",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Wide Latin")))
+                                   label=UN(_("Wide Latin"))))
 
         props[self.__input_mode].set_state(ibus.PROP_STATE_CHECKED)
 
@@ -163,19 +178,19 @@ class Engine(ibus.EngineBase):
         typing_mode_prop = ibus.Property(key=u"TypingMode",
                                          type=ibus.PROP_TYPE_MENU,
                                          label=u"R",
-                                         tooltip=_(u"Switch typing mode"))
+                                         tooltip=UN(_("Switch typing mode")))
         self.__prop_dict[u"TypingMode"] = typing_mode_prop
 
         props = ibus.PropList()
         props.append(ibus.Property(key=u"TypingMode.Romaji",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Romaji")))
+                                   label=UN(_("Romaji"))))
         props.append(ibus.Property(key=u"TypingMode.Kana",
                                    type=ibus.PROP_TYPE_RADIO,
-                                   label=_(u"Kana")))
-        # props.append(ibus.Property(name = u"TypingMode.ThumbShift",
-        #                     type = ibus.PROP_TYPE_RADIO,
-        #                     label = _(u"Thumb shift")))
+                                   label=UN(_("Kana"))))
+        props.append(ibus.Property(key=u"TypingMode.ThumbShift",
+                                   type=ibus.PROP_TYPE_RADIO,
+                                   label=UN(_("Thumb shift"))))
         props[Engine.__typing_mode].set_state(ibus.PROP_STATE_CHECKED)
 
         for prop in props:
@@ -184,11 +199,46 @@ class Engine(ibus.EngineBase):
         typing_mode_prop.set_sub_props(props)
         anthy_props.append(typing_mode_prop)
 
+        self.__set_dict_props(anthy_props)
         anthy_props.append(ibus.Property(key=u"setup",
-                                         tooltip=_(u"Configure Anthy")))
+                                         tooltip=UN(_("Configure Anthy"))))
 
         return anthy_props
 
+    def __set_dict_props(self, anthy_props):
+        path = self.__prefs.get_value('common', 'dict_admin_command')
+        if not os.path.exists(path[0]):
+            return
+
+        if os.path.exists(KASUMI_IMG_PATH):
+            label = u""
+            icon = unicode(KASUMI_IMG_PATH)
+        else:
+            # Translators: "Dic" means 'dictionary', One kanji may be good.
+            label = UN(_("Dic"))
+            icon = u""
+
+        dict_prop = ibus.Property(key=u"setup-dict-kasumi",
+                                  type=ibus.PROP_TYPE_MENU,
+                                  label=label,
+                                  icon=icon,
+                                  tooltip=UN(_("Configure dictionaries")))
+
+        props = ibus.PropList()
+        props.append(ibus.Property(key=u"setup-dict-kasumi-admin",
+                                   type=ibus.PROP_TYPE_NORMAL,
+                                   label=UN(_("Edit dictionaries")),
+                                   icon=icon,
+                                   tooltip=UN(_("Launch the dictionary tool"))))
+        props.append(ibus.Property(key=u"setup-dict-kasumi-word",
+                                   type=ibus.PROP_TYPE_NORMAL,
+                                   label=UN(_("Add words")),
+                                   icon=icon,
+                                   tooltip=UN(_("Add words in the dictionary"))))
+
+        dict_prop.set_sub_props(props)
+        anthy_props.append(dict_prop)
+
     def __get_clipboard(self, clipboard, text, data):
         clipboard_text = clipboard.wait_for_text ()
 
@@ -304,7 +354,7 @@ class Engine(ibus.EngineBase):
                 traceback.print_exc()
                 return False
 
-    def process_key_event_internal(self, keyval, state):
+    '''def process_key_event_internal(self, keyval, state):
         is_press = (state & modifier.RELEASE_MASK) == 0
 
         state = state & (modifier.SHIFT_MASK |
@@ -371,7 +421,7 @@ class Engine(ibus.EngineBase):
         else:
             if not self.__preedit_ja_string.is_empty():
                 return True
-            return False
+            return False'''
 
     def property_activate(self, prop_name, state):
 
@@ -383,6 +433,10 @@ class Engine(ibus.EngineBase):
         else:
             if prop_name == 'setup':
                 self.__start_setup()
+            elif prop_name == 'setup-dict-kasumi-admin':
+                self.__start_dict_admin()
+            elif prop_name == 'setup-dict-kasumi-word':
+                self.__start_add_word()
             else:
                 self.__prop_dict[prop_name].set_state(state)
 
@@ -402,6 +456,7 @@ class Engine(ibus.EngineBase):
             print >> sys.stderr, "Unknow prop_name = %s" % prop_name
             return True
         self.__prop_dict[prop_name].set_state(state)
+        self.update_property(self.__prop_dict[prop_name])
 
         mode, label = input_modes[prop_name]
         if self.__input_mode == mode:
@@ -422,12 +477,14 @@ class Engine(ibus.EngineBase):
         typing_modes = {
             u"TypingMode.Romaji" : (jastring.TYPING_MODE_ROMAJI, u"R"),
             u"TypingMode.Kana" : (jastring.TYPING_MODE_KANA, u"か"),
+            u"TypingMode.ThumbShift" : (jastring.TYPING_MODE_THUMB_SHIFT, u"親"),
         }
 
         if prop_name not in typing_modes:
             print >> sys.stderr, "Unknow prop_name = %s" % prop_name
             return True
         self.__prop_dict[prop_name].set_state(state)
+        self.update_property(self.__prop_dict[prop_name])
 
         mode, label = typing_modes[prop_name]
 
@@ -444,6 +501,7 @@ class Engine(ibus.EngineBase):
         modes = {
             jastring.TYPING_MODE_ROMAJI : (u"TypingMode.Romaji", u"R"),
             jastring.TYPING_MODE_KANA : (u"TypingMode.Kana", u"か"),
+            jastring.TYPING_MODE_THUMB_SHIFT : (u"TypingMode.ThumbShift", u"親"),
         }
         prop_name, label = modes.get(Engine.__typing_mode, (None, None))
         if prop_name == None or label == None:
@@ -461,7 +519,7 @@ class Engine(ibus.EngineBase):
 #        self.__invalidate()
 
     def focus_out(self):
-        mode = self.__prefs.get_value('common', 'behivior_on_focus_out')
+        mode = self.__prefs.get_value('common', 'behavior_on_focus_out')
         if mode == 0:
             self.__reset()
             self.__invalidate()
@@ -698,7 +756,7 @@ class Engine(ibus.EngineBase):
         self.__invalidate()
         return True
 
-    def __on_key_hiragana_katakana(self):
+    '''def __on_key_hiragana_katakana(self):
         if self.__convert_mode == CONV_MODE_ANTHY:
             self.__end_anthy_convert()
 
@@ -717,9 +775,9 @@ class Engine(ibus.EngineBase):
         self.update_property(prop)
 
         self.__invalidate()
-        return True
+        return True'''
 
-    def __on_key_muhenka(self):
+    '''def __on_key_muhenka(self):
         if self.__preedit_ja_string.is_empty():
             return False
 
@@ -735,9 +793,9 @@ class Engine(ibus.EngineBase):
 
         self.__invalidate()
 
-        return True
+        return True'''
 
-    def __on_key_henkan(self):
+    '''def __on_key_henkan(self):
         if self.__preedit_ja_string.is_empty():
             return False
         if self.__convert_mode != CONV_MODE_ANTHY:
@@ -746,9 +804,9 @@ class Engine(ibus.EngineBase):
         elif self.__convert_mode == CONV_MODE_ANTHY:
             self.__lookup_table_visible = True
             self.cursor_down()
-        return True
+        return True'''
 
-    def __on_key_space(self, wide=False):
+    '''def __on_key_space(self, wide=False):
         if self.__input_mode == INPUT_MODE_WIDE_LATIN or wide:
             # Input Wide space U+3000
             wide_char = symbol_rule[unichr(keysyms.space)]
@@ -772,7 +830,7 @@ class Engine(ibus.EngineBase):
         elif self.__convert_mode == CONV_MODE_ANTHY:
             self.__lookup_table_visible = True
             self.cursor_down()
-        return True
+        return True'''
 
     def __on_key_up(self):
         if self.__preedit_ja_string.is_empty():
@@ -802,7 +860,7 @@ class Engine(ibus.EngineBase):
             self.page_down()
         return True
 
-    def __on_key_left(self):
+    '''def __on_key_left(self):
         if self.__preedit_ja_string.is_empty():
             return False
 
@@ -820,7 +878,7 @@ class Engine(ibus.EngineBase):
         self.__lookup_table_visible = False
         self.__fill_lookup_table()
         self.__invalidate()
-        return True
+        return True'''
 
     def __on_key_right(self):
         if self.__preedit_ja_string.is_empty():
@@ -941,10 +999,15 @@ class Engine(ibus.EngineBase):
     @classmethod
     def CONFIG_VALUE_CHANGED(cls, bus, section, name, value):
         print 'VALUE_CHAMGED =', section, name, value
-        section = section[len(cls.__prefs._prefix) + 1:]
-        if section == cls._get_shortcut_type():
+
+        if not section.startswith("engine/anthy"):
+            # This value is used for ibus.config.set_value only.
+            return
+
+        base_sec = section[len(cls.__prefs._prefix) + 1:]
+        sec = cls._get_shortcut_type()
+        if base_sec == sec:
             cmd = '_Engine__cmd_' + name
-            sec = cls._get_shortcut_type()
             old = cls.__prefs.get_value(sec, name)
             value = value if value != [''] else []
             for s in set(old).difference(value):
@@ -957,12 +1020,14 @@ class Engine(ibus.EngineBase):
                     lambda a, b: cmp(keys.index(a[13:]), keys.index(b[13:])))
 
             cls.__prefs.set_value(sec, name, value)
-        elif section == 'common':
-            cls.__prefs.set_value(section, name, value)
+        elif base_sec == 'common':
+            cls.__prefs.set_value(base_sec, name, value)
             if name == 'shortcut_type':
                 cls.__keybind = cls._mk_keybind()
+        elif base_sec:
+            cls.__prefs.set_value(base_sec, name, value)
         else:
-            cls.__prefs.set_value(sec, name, value)
+            cls.__prefs.set_value(section, name, value)
 
     @classmethod
     def _mk_keybind(cls):
@@ -1001,7 +1066,126 @@ class Engine(ibus.EngineBase):
 
         return repr([int(state), int(keyval)])
 
+    def process_key_event_thumb(self, keyval, keycode, state):
+        import gtk
+        import thumb
+
+        def on_timeout(keyval):
+            if self._MM:
+                insert(thumb.table[self._MM][self._SS])
+            else:
+                cmd_exec([0, RS(), LS()][self._SS])
+            self._H = None
+
+        def start(t):
+            self._H = gobject.timeout_add(t, on_timeout, keyval)
+
+        def stop():
+            if self._H:
+                gobject.source_remove(self._H)
+                self._H = None
+                return True
+            return False
+
+        def insert(keyval):
+            try:
+                ret = self.__on_key_common(ord(keyval))
+                if (keyval in u',.、。' and
+                    self.__prefs.get_value('common', 'behavior_on_period')):
+                    return self.__cmd_convert(keyval, state)
+                return ret
+            except:
+                pass
+
+        def cmd_exec(keyval, state=0):
+            key = self._mk_key(keyval, state)
+            for cmd in self.__keybind.get(key, []):
+                print 'cmd =', cmd
+                try:
+                    if getattr(self, cmd)(keyval, state):
+                        return True
+                except:
+                    print >> sys.stderr, 'Unknow command = %s' % cmd
+            return False
+
+        def RS():
+            return self.__prefs.get_value('common', 'thumb_rs')
+
+        def LS():
+            return self.__prefs.get_value('common', 'thumb_ls')
+
+        def T1():
+            return self.__prefs.get_value('common', 'thumb_t1')
+
+        def T2():
+            return self.__prefs.get_value('common', 'thumb_t2')
+
+        state = state & (modifier.SHIFT_MASK |
+                         modifier.CONTROL_MASK |
+                         modifier.MOD1_MASK |
+                         modifier.RELEASE_MASK)
+
+        if keyval in KP_Table and self.__prefs.get_value('common',
+                                                         'ten_key_mode'):
+            keyval = KP_Table[keyval]
+
+        if state & modifier.RELEASE_MASK:
+            if keyval == self._MM:
+                if stop():
+                    insert(thumb.table[self._MM][self._SS])
+                self._MM = 0
+            elif (1 if keyval == RS() else 2) == self._SS:
+                if stop():
+                    cmd_exec([0, RS(), LS()][self._SS])
+                self._SS = 0
+        else:
+            if keyval in [LS(), RS()] and state == 0:
+                if self._SS:
+                    stop()
+                    cmd_exec([0, RS(), LS()][self._SS])
+                    self._SS = 1 if keyval == RS() else 2
+                    start(T1())
+                elif self._MM:
+                    stop()
+                    insert(thumb.table[self._MM][1 if keyval == RS() else 2])
+                else:
+                    self._SS = 1 if keyval == RS() else 2
+                    start(T1())
+            elif keyval in thumb.table.keys() and state == 0:
+                if self._MM:
+                    stop()
+                    insert(thumb.table[self._MM][self._SS])
+                    start(T2())
+                    self._MM = keyval
+                elif self._SS:
+                    stop()
+                    insert(thumb.table[keyval][self._SS])
+                else:
+                    if cmd_exec(keyval, state):
+                        return True
+                    start(T2())
+                    self._MM = keyval
+            else:
+                if self._MM:
+                    stop()
+                    insert(thumb.table[self._MM][self._SS])
+                elif self._SS:
+                    stop()
+                    cmd_exec([0, RS(), LS()][self._SS])
+                if cmd_exec(keyval, state):
+                    return True
+                elif 0x21 <= keyval <= 0x7e and state & (modifier.CONTROL_MASK | modifier.MOD1_MASK) == 0:
+                    insert(thumb.shift_table.get(keyval, unichr(keyval)))
+                else:
+                    if not self.__preedit_ja_string.is_empty():
+                        return True
+                    return False
+        return True
+
     def process_key_event_internal2(self, keyval, keycode, state):
+        if self.__typing_mode == jastring.TYPING_MODE_THUMB_SHIFT and \
+           self.__input_mode not in [INPUT_MODE_LATIN, INPUT_MODE_WIDE_LATIN]:
+            return self.process_key_event_thumb(keyval, keycode, state)
 
         is_press = (state & modifier.RELEASE_MASK) == 0
 
@@ -1038,7 +1222,7 @@ class Engine(ibus.EngineBase):
                     keyval = keysyms.yen
             ret = self.__on_key_common(keyval)
             if (unichr(keyval) in u',.' and
-                self.__prefs.get_value('common', 'behivior_on_period')):
+                self.__prefs.get_value('common', 'behavior_on_period')):
                 return self.__cmd_convert(keyval, state)
             return ret
         else:
@@ -1126,7 +1310,8 @@ class Engine(ibus.EngineBase):
             return False
 
         modes = {
-            jastring.TYPING_MODE_KANA: u"TypingMode.Romaji",
+            jastring.TYPING_MODE_THUMB_SHIFT: u"TypingMode.Romaji",
+            jastring.TYPING_MODE_KANA: u"TypingMode.ThumbShift",
             jastring.TYPING_MODE_ROMAJI: u"TypingMode.Kana",
         }
         self.__typing_mode_activate(modes[self.__typing_mode],
@@ -1135,18 +1320,20 @@ class Engine(ibus.EngineBase):
 
     #edit_keys
     def __cmd_insert_space(self, keyval, state):
-        if self.__input_mode in [INPUT_MODE_LATIN,
-                                 INPUT_MODE_HALF_WIDTH_KATAKANA]:
+        if (self.__prefs.get_value('common', 'half_width_space') or
+            self.__input_mode in [INPUT_MODE_LATIN,
+                                  INPUT_MODE_HALF_WIDTH_KATAKANA]):
             return self.__cmd_insert_half_space(keyval, state)
         else:
             return self.__cmd_insert_wide_space(keyval, state)
 
     def __cmd_insert_alternate_space(self, keyval, state):
-        if not self.__input_mode in [INPUT_MODE_LATIN,
-                                     INPUT_MODE_HALF_WIDTH_KATAKANA]:
-            return self.__cmd_insert_half_space(keyval, state)
-        else:
+        if (self.__prefs.get_value('common', 'half_width_space') or
+            self.__input_mode in [INPUT_MODE_LATIN,
+                                  INPUT_MODE_HALF_WIDTH_KATAKANA]):
             return self.__cmd_insert_wide_space(keyval, state)
+        else:
+            return self.__cmd_insert_half_space(keyval, state)
 
     def __cmd_insert_half_space(self, keyval, state):
         if not self._chk_mode('0'):
@@ -1366,7 +1553,7 @@ class Engine(ibus.EngineBase):
             text, cursor = self.__get_preedit()
             commit_length = 0
             for i in xrange(0, commit_index + 1):
-                buf = self.__context.get_segment(i, -1)
+                buf = self.__context.get_segment(i, NTH_UNCONVERTED_CANDIDATE)
                 commit_length += len(unicode(buf, "utf-8"))
             self.__preedit_ja_string.move_cursor(commit_length - cursor)
             for i in xrange(0, commit_length):
@@ -1486,8 +1673,21 @@ class Engine(ibus.EngineBase):
 
     #convert_keys
     def __cmd_convert_to_char_type_forward(self, keyval, state):
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            n = self.__segments[self.__cursor_pos][0]
+            if n == NTH_HIRAGANA_CANDIDATE:
+                return self.__convert_segment_to_kana(NTH_KATAKANA_CANDIDATE)
+            elif n == NTH_KATAKANA_CANDIDATE:
+                return self.__convert_segment_to_kana(NTH_HALFKANA_CANDIDATE)
+            elif n == NTH_HALFKANA_CANDIDATE:
+                return self.__convert_segment_to_latin(-100)
+            elif n == -100:
+                return self.__convert_segment_to_latin(-101)
+            else:
+                return self.__convert_segment_to_kana(NTH_HIRAGANA_CANDIDATE)
+
         if self.__convert_mode == CONV_MODE_KATAKANA:
-            return self.__cmd_convert_to_half(keyval, state)
+            return self.__cmd_convert_to_half_katakana(keyval, state)
         elif self.__convert_mode == CONV_MODE_HALF_WIDTH_KATAKANA:
             return self.__cmd_convert_to_latin(keyval, state)
         elif CONV_MODE_LATIN_0 <= self.__convert_mode <= CONV_MODE_LATIN_3:
@@ -1499,52 +1699,126 @@ class Engine(ibus.EngineBase):
             return self.__cmd_convert_to_katakana(keyval, state)
 
     def __cmd_convert_to_char_type_backward(self, keyval, state):
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            n = self.__segments[self.__cursor_pos][0]
+            if n == NTH_KATAKANA_CANDIDATE:
+                return self.__convert_segment_to_kana(NTH_HIRAGANA_CANDIDATE)
+            elif n == NTH_HALFKANA_CANDIDATE:
+                return self.__convert_segment_to_kana(NTH_KATAKANA_CANDIDATE)
+            elif n == -100:
+                return self.__convert_segment_to_kana(NTH_HALFKANA_CANDIDATE)
+            elif n == -101:
+                return self.__convert_segment_to_latin(-100)
+            else:
+                return self.__convert_segment_to_latin(-101)
+
         if self.__convert_mode == CONV_MODE_KATAKANA:
             return self.__cmd_convert_to_hiragana(keyval, state)
         elif self.__convert_mode == CONV_MODE_HALF_WIDTH_KATAKANA:
             return self.__cmd_convert_to_katakana(keyval, state)
         elif CONV_MODE_LATIN_0 <= self.__convert_mode <= CONV_MODE_LATIN_3:
-            return self.__cmd_convert_to_half(keyval, state)
+            return self.__cmd_convert_to_half_katakana(keyval, state)
         elif (CONV_MODE_WIDE_LATIN_0 <= self.__convert_mode
                                      <= CONV_MODE_WIDE_LATIN_3):
             return self.__cmd_convert_to_latin(keyval, state)
         else:
             return self.__cmd_convert_to_wide_latin(keyval, state)
 
+    def __convert_segment_to_kana(self, n):
+        if self.__convert_mode == CONV_MODE_ANTHY and -4 <= n <= -2:
+            buf = self.__context.get_segment(self.__cursor_pos, n)
+            self.__segments[self.__cursor_pos] = n, unicode(buf, "utf-8")
+            self.__lookup_table_visible = False
+            self.__invalidate()
+            return True
+
+        return False
+
     def __cmd_convert_to_hiragana(self, keyval, state):
-        if not self._chk_mode('1234'):
+        if not self._chk_mode('12345'):
             return False
 
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            return self.__convert_segment_to_kana(NTH_HIRAGANA_CANDIDATE)
+
         return self.__on_key_conv(0)
 
     def __cmd_convert_to_katakana(self, keyval, state):
-        if not self._chk_mode('1234'):
+        if not self._chk_mode('12345'):
             return False
 
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            return self.__convert_segment_to_kana(NTH_KATAKANA_CANDIDATE)
+
         return self.__on_key_conv(1)
 
     def __cmd_convert_to_half(self, keyval, state):
-        if not self._chk_mode('1234'):
+        if not self._chk_mode('12345'):
             return False
 
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            i, s = self.__segments[self.__cursor_pos]
+            if i == -101:
+                return self.__convert_segment_to_latin(-100)
+            elif i == -100:
+                return self.__convert_segment_to_latin(-100)
+            return self.__convert_segment_to_kana(NTH_HALFKANA_CANDIDATE)
+
+        elif CONV_MODE_WIDE_LATIN_0 <= self.__convert_mode <= CONV_MODE_WIDE_LATIN_3:
+            return self.__on_key_conv(4)
+        elif CONV_MODE_LATIN_0 <= self.__convert_mode <= CONV_MODE_LATIN_3:
+            return self.__on_key_conv(4)
         return self.__on_key_conv(2)
 
     def __cmd_convert_to_half_katakana(self, keyval, state):
-        if not self._chk_mode('1234'):
+        if not self._chk_mode('12345'):
             return False
 
+        if self.__convert_mode == CONV_MODE_ANTHY:
+            return self.__convert_segment_to_kana(NTH_HALFKANA_CANDIDATE)
+
         return self.__on_key_conv(2)
 
+    def __convert_segment_to_latin(self, n):
+        if self.__convert_mode == CONV_MODE_ANTHY and n in [-100, -101]:
+            start = 0
+            for i in range(self.__cursor_pos):
+                start += len(unicode(self.__context.get_segment(i, NTH_UNCONVERTED_CANDIDATE), 'utf-8'))
+            end = start + len(unicode(self.__context.get_segment(self.__cursor_pos, NTH_UNCONVERTED_CANDIDATE), 'utf-8'))
+            i, s = self.__segments[self.__cursor_pos]
+            s2 = self.__preedit_ja_string.get_raw(start, end)
+            if n == -101:
+                s2 = u''.join([unichar_half_to_full(c) for c in s2])
+            if i == n:
+                if s == s2.lower():
+                    s2 = s2.upper()
+                elif s == s2.upper():
+                    s2 = s2.capitalize()
+                elif s == s2 or s == s2.capitalize():
+                    s2 = s2.lower()
+            self.__segments[self.__cursor_pos] = n, s2
+            self.__lookup_table_visible = False
+            self.__invalidate()
+            return True
+
+        return False
+
     def __cmd_convert_to_wide_latin(self, keyval, state):
-        if not self._chk_mode('1234'):
+        if not self._chk_mode('12345'):
             return False
 
+        if self.__convert_mode == CONV_MODE_ANTHY:
+           return self.__convert_segment_to_latin(-101)
+
         return self.__on_key_conv(3)
 
     def __cmd_convert_to_latin(self, keyval, state):
-        if not self._chk_mode('1234'):
+        if not self._chk_mode('12345'):
             return False
 
+        if self.__convert_mode == CONV_MODE_ANTHY:
+           return self.__convert_segment_to_latin(-100)
+
         return self.__on_key_conv(4)
 
     #dictonary_keys
@@ -1552,16 +1826,14 @@ class Engine(ibus.EngineBase):
         if not self._chk_mode('0'):
             return False
 
-        path = self.__prefs.get_value('common', 'dict_admin_command')
-        os.spawnl(os.P_NOWAIT, *path)
+        self.__start_dict_admin()
         return True
 
     def __cmd_add_word(self, keyval, state):
         if not self._chk_mode('0'):
             return False
 
-        path = self.__prefs.get_value('common', 'add_word_command')
-        os.spawnl(os.P_NOWAIT, *path)
+        self.__start_add_word()
         return True
 
     def __cmd_start_setup(self, keyval, state):
@@ -1571,6 +1843,14 @@ class Engine(ibus.EngineBase):
         self.__start_setup()
         return True
 
+    def __start_dict_admin(self):
+        path = self.__prefs.get_value('common', 'dict_admin_command')
+        os.spawnl(os.P_NOWAIT, *path)
+
+    def __start_add_word(self):
+        path = self.__prefs.get_value('common', 'add_word_command')
+        os.spawnl(os.P_NOWAIT, *path)
+
     def __start_setup(self):
         if Engine.__setup_pid != 0:
             pid, state = os.waitpid(Engine.__setup_pid, os.P_NOWAIT)
