@@ -3,7 +3,8 @@
 #
 # ibus-anthy - The Anthy engine for IBus
 #
-# Copyright (c) 2007-2008 Huang Peng <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,20 +22,27 @@
 
 import romaji
 import kana
+import thumb
 
 from segment import unichar_half_to_full
 
-SymbolTable = {}
+HalfSymbolTable = {}
 for i in range(32, 127):
     if not chr(i).isalnum():
-        SymbolTable[unichar_half_to_full(chr(i))] = chr(i)
+        HalfSymbolTable[unichar_half_to_full(chr(i))] = chr(i)
 
-NumberTable = {}
+HalfNumberTable = {}
 for i in range(10):
-    NumberTable[unichar_half_to_full(str(i))] = str(i)
+    HalfNumberTable[unichar_half_to_full(str(i))] = str(i)
 
 PeriodTable = {u'。': u'．', u'、': u'，', u'｡': u'.', u'､': u','}
 
+SymbolTable = {}
+SymbolTable[0] = {u'「': u'「', u'」': u'」', u'／': u'／'}
+SymbolTable[1] = {u'「': u'「', u'」': u'」', u'／': u'・'}
+SymbolTable[2] = {u'「': u'［', u'」': u'］', u'／': u'／'}
+SymbolTable[3] = {u'「': u'［', u'」': u'］', u'／': u'・'}
+
 TYPING_MODE_ROMAJI, \
 TYPING_MODE_KANA, \
 TYPING_MODE_THUMB_SHIFT = range(3)
@@ -71,6 +79,8 @@ class JaString:
                     new_segments = [romaji.RomajiSegment(c)]
                 elif self.__mode == TYPING_MODE_KANA:
                     new_segments = [kana.KanaSegment(c)]
+                elif self.__mode == TYPING_MODE_THUMB_SHIFT:
+                    new_segments = [thumb.ThumbShiftSegment(c)]
         if new_segments:
             self.__segments[self.__cursor:self.__cursor] = new_segments
             self.__cursor += len(new_segments)
@@ -110,13 +120,15 @@ class JaString:
 
     def _chk_text(self, s):
         period = self._prefs.get_value('common', 'period_style')
-        symbol = self._prefs.get_value('common', 'half_width_symbol')
-        number = self._prefs.get_value('common', 'half_width_number')
+        symbol = self._prefs.get_value('common', 'symbol_style')
+        half_symbol = self._prefs.get_value('common', 'half_width_symbol')
+        half_number = self._prefs.get_value('common', 'half_width_number')
         ret = ''
         for c in s:
             c = c if not period else PeriodTable.get(c, c)
-            c = c if not symbol else SymbolTable.get(c, c)
-            c = c if not number else NumberTable.get(c, c)
+            c = c if not symbol else SymbolTable[symbol].get(c, c)
+            c = c if not half_symbol else HalfSymbolTable.get(c, c)
+            c = c if not half_number else HalfNumberTable.get(c, c)
             ret += c
         return ret
 
@@ -156,3 +168,13 @@ class JaString:
     def is_empty(self):
         return all(map(lambda s: s.is_empty(), self.__segments))
 
+    def get_raw(self, start, end):
+        i = 0
+        r = u''
+        for s in self.__segments:
+            if i >= end:
+                break
+            elif start <= i:
+                r += s.to_latin()
+            i += len(s.to_hiragana())
+        return r
