@@ -1,5 +1,28 @@
 # -*- coding: utf-8 -*-
+# vim:set noet ts=4:
+#
+# ibus-anthy - The Anthy engine for IBus
+#
+# Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
+import gtk
+import sys
 
 from prefs import Prefs
 
@@ -14,8 +37,40 @@ class AnthyPrefs(Prefs):
         super(AnthyPrefs, self).__init__(bus, config)
         self.default = _config
 
+        # The keys will be EOSL in the near future.
+        self.__update_key ("common",
+                           "behivior_on_focus_out",
+                           "behavior_on_focus_out")
+        self.__update_key ("common",
+                           "behivior_on_period",
+                           "behavior_on_period")
         self.fetch_all()
 
+    def __update_key (self, section, old_key, new_key):
+        file = __file__
+        path_list = __file__.split('/')
+        if path_list:
+            path_list.reverse()
+            file = path_list[0]
+        warning_message = \
+            "(" + file + ") ibus-anthy-WARNING **: "                        \
+            "The key (" + old_key + ") will be removed in the future. "     \
+            "Currently the key (" + new_key + ") is used instead. "         \
+            "The ibus keys are defined in " +                               \
+            "/".join(["/desktop/ibus", self._prefix, section]) + " ."
+
+        if not self.fetch_item(section, old_key, True):
+            return
+        print >> sys.stderr, warning_message
+        if self.fetch_item(section, new_key, True):
+            return
+
+        self.fetch_item(section, old_key)
+        value = self.get_value(section, old_key)
+        self.set_value(section, new_key, value)
+        self.commit_item(section, new_key)
+        self.undo_item(section, new_key)
+
     def keys(self, section):
         if section.startswith('shortcut/'):
             return _cmd_keys
@@ -101,18 +156,25 @@ _config = {
         'typing_method': 0,
 
         'period_style': 0,
-        'ten_key_mode': 0,
-        'behivior_on_focus_out': 0,
-        'behivior_on_period': 0,
+        'symbol_style': 1,
+        'ten_key_mode': 1,
+        'behavior_on_focus_out': 0,
+        'behavior_on_period': 0,
 
         'page_size': 10,
         'half_width_symbol': False,
         'half_width_number': False,
+        'half_width_space': False,
 
         'shortcut_type': 'default',
 
         'dict_admin_command': ['/usr/bin/kasumi', 'kasumi'],
         'add_word_command': ['/usr/bin/kasumi', 'kasumi', '-a'],
+
+        'thumb_rs': gtk.keysyms.Henkan,
+        'thumb_ls': gtk.keysyms.Muhenkan,
+        'thumb_t1': 100,
+        'thumb_t2': 75,
     }
 }
 
