@@ -1,9 +1,31 @@
-from os import path
+# vim:set noet ts=4:
+#
+# ibus-anthy - The Anthy engine for IBus
+#
+# Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
+# Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
+# Copyright (c) 2007-2010 Red Hat, Inc.
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2, or (at your option)
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+from os import path, getenv
 import gtk
 import pango
 from gtk import glade
 from ibus import keysyms, modifier
-from gettext import dgettext
+from gettext import dgettext, bindtextdomain
 
 from anthyprefs import AnthyPrefs
 
@@ -21,14 +43,27 @@ class AnthySetup(object):
     def __init__(self):
         self.prefs = prefs = AnthyPrefs()
 
+        localedir = getenv("IBUS_LOCALEDIR")
+        bindtextdomain("ibus-anthy", localedir)
+        glade.bindtextdomain("ibus-anthy", localedir)
         glade.textdomain("ibus-anthy")
         glade_file = path.join(path.dirname(__file__), "setup.glade")
         self.xml = xml = glade.XML(glade_file)
 
+        # glade "icon_name" property has a custom scaling and it seems
+        # to be difficult to show the complicated small icon in metacity.
+        # This can add the pixbuf without scaling.
+        anthydir = path.dirname(path.dirname(__file__))
+        if not anthydir:
+            anthydir = "/usr/share/ibus-anthy"
+        icon_path = path.join(anthydir, "icons", "ibus-anthy.png")
+        if path.exists(icon_path):
+            xml.get_widget('main').set_icon_from_file(icon_path)
+
         for name in ['input_mode', 'typing_method',
-                     'period_style', 'ten_key_mode',
-                     'behivior_on_focus_out', 'behivior_on_period',
-                     'half_width_symbol', 'half_width_number']:
+                     'period_style', 'symbol_style', 'ten_key_mode',
+                     'behavior_on_focus_out', 'behavior_on_period',
+                     'half_width_symbol', 'half_width_number', 'half_width_space']:
             xml.get_widget(name).set_active(prefs.get_value('common', name))
 
         l = ['default', 'atok', 'wnn']
@@ -40,11 +75,11 @@ class AnthySetup(object):
                                                               'page_size'))
 
         tv = xml.get_widget('shortcut')
-        tv.append_column(gtk.TreeViewColumn('Command',
+        tv.append_column(gtk.TreeViewColumn(_("Command"),
                                              gtk.CellRendererText(), text=0))
         renderer = gtk.CellRendererText()
         renderer.set_property("ellipsize", pango.ELLIPSIZE_END)
-        tv.append_column(gtk.TreeViewColumn('Shortcut',
+        tv.append_column(gtk.TreeViewColumn(_("Shortcut"),
                                              renderer, text=1))
         tv.get_selection().connect_after('changed',
                                           self.on_selection_changed, 0)
@@ -147,7 +182,7 @@ class AnthySetup(object):
     def _get_shortcut_sec(self):
         l = ['default', 'atok', 'wnn']
         s_type = self.xml.get_widget('shortcut_type').get_active_text().lower()
-        return 'shortcut/' + s_type if s_type in l else 'default'
+        return 'shortcut/' + (s_type if s_type in l else 'default')
 
     def on_shortcut_type_changed(self, widget):
         ls = self.xml.get_widget('shortcut').get_model()
@@ -192,7 +227,7 @@ class AnthySetup(object):
     def on_button7_clicked(self, widget):
         dlg = self.xml.get_widget('key_input_dialog')
         dlg.set_markup('<big><b>%s</b></big>' % _('Please press a key (or a key combination)'))
-        dlg.format_secondary_text(_('The dialog willbe closed when the key is released'))
+        dlg.format_secondary_text(_('The dialog will be closed when the key is released'))
         id = dlg.run()
         dlg.hide()
         if id == gtk.RESPONSE_OK:
